"""
Main loop / pseudo code:

in every case, if No, continue with next loop:
    - shutdown PC
    - start PC via remote PC
    - check if cam in device manager   
    - wait 10 secs for user input, if yes, terminate program

"""

import sys
import os
import time
import win32com.client
import subprocess
from datetime import datetime
from jsonconf.data import JsonConf

LOG_FILE = os.path.join(__file__, "../log.txt")

class RestartTest:
    def __init__(self):
        self.log = TextLog()
        conf_filename = os.path.join(__file__, "../usb_reboot.jsonconf")
        self.conf = JsonConf(conf_filename).as_dict()

    def get_loop_count(self, filename=None):
        """ Pulls loop count number out of text file or returns 0 on any kind of error. """
        if not filename:
            filename = os.path.join(__file__, "../loop_count.txt")
        try:
            with open(filename, "r") as f:
                return int(f.read().strip())
        except:
            return 0

    def save_loop_count(self, loop_count, filename=None):
        if not filename:
            filename = os.path.join(__file__, "../loop_count.txt")
        with open(filename, "w") as f:
            f.write("{:d}".format(loop_count))

    def run(self):
        loop_count = self.get_loop_count()
        loop_count_max = self.conf["loop_count_max"]
        if loop_count_max != 0:
            loop_count_max += 1   # 1st loop is done without reboot before so this one doesn't count

        self.log("Testdurchlauf #{}".format(loop_count))
        if (not self.cams_in_device_manager()):
            self.stop("Fehlerfall - Programm gestoppt.")
        else:
            loop_count += 1    # count number of succeeded loops
            self.save_loop_count(loop_count)
            if 0 == loop_count_max or loop_count_max > loop_count:
                self.restart()
            else:
                self.stop("Maximale Loop-Anzahl erreicht. Test gestoppt.")

    def stop(self, stop_msg=None):
        """ End of test. """
        if stop_msg:  self.log(stop_msg)
        self.log.save()
        os.startfile(LOG_FILE)
        # ret = subprocess.call("devcon.exe disable *VEN_1912*")   # disable root hub for LeCroy

    def cams_in_device_manager(self):
        """ Returns True if num_cameras devices are found in WMI 'PnpEntities' which have conf's model_name(s). """
        retry_count = 0
        while True:
            cam_count = 0
            wmi = win32com.client.GetObject("winmgmts:")
            for device in wmi.InstancesOf("Win32_PnPEntity"):
                if device.Caption in self.conf["model_names"]:
                    self.log("Kamera im Gerätemanager gefunden.")
                    cam_count += 1
                    if cam_count == self.conf["num_cameras"]:
                        return True
            self.log("---> Nicht alle {} Kamera(s) im Gerätemanager gefunden.".format(self.conf["num_cameras"]))
            if retry_count >= self.conf["device_manager.retries"]:
                break
            else:
                self.log("Suche erneut.")
            retry_count += 1
            time.sleep(self.conf["device_manager.retry_delay"])
        # we didnt find all cams, but still return True if testing for a specific serial (done in bgapy later)
        return False or bool(self.conf["serial"])
		
    def restart(self):
        """ Exit the Python process and shutdown system if not aborted by user."""
        if self.conf["restart_type"] == "Reboot":
            cmd = "shutdown -t 0 -r -f"     # no waiting, reboot, forced
            msg = "System wird neugestartet."
        elif self.conf["restart_type"] == "Shutdown":
            cmd = "shutdown -t 0 -s -f"     # no waiting, shutdown, forced
            msg = "System wird heruntergefahren."
        elif self.conf["restart_type"] == "Hibernate":
            cmd = "shutdown -h -f"          # hibernate, forced. must not use -t along with it.
            msg = "System wird in Ruhezustand versetzt."
        elif self.conf["restart_type"] == "Nothing":
            cmd = "ping -n 1 127.0.0.1"   #  dummy command
            msg = "Programm wird beendet."

        print("{} Abbruch durch Ctrl+C.".format(msg))
        try:
            ret = subprocess.call("timeout /T 3")
        except KeyboardInterrupt:
            ret = 1
        if ret == 0:    #  no CTRL-C ?
            self.log("{}!\n".format(self.conf["restart_type"]))
            self.log.save()
            time.sleep(1)
            subprocess.call(cmd)
            sys.exit()
        else:
            self.stop("{} abgebrochen durch Nutzer.\n".format(self.conf["restart_type"]))

class TextLog():
    timestamp_format = "%Y-%m-%d %H:%M:%S"

    def __init__(self):
        self.filename = LOG_FILE
        self.txt = ""
        self.save()

    def log_text(self, txt, end="\n", with_timestamp=True):
        """Logs txt with preceeding timestamp. Also prints to console.

        Parameters:
        --------------
        end : str
            Is appended to txt. Linebreak is default.
        with_timestamp : bool
            Precede with timestamp or not.
        """
        if with_timestamp:
            output = "{} {}{}".format(datetime.now().strftime(self.timestamp_format), txt, end)
        else:
            output = "{}{}".format(txt, end)
        print(output, flush=True, end="")
        self.txt += output

    __call__ = log_text    # redirect calls of self to log_text

    def save(self):
        with open(self.filename, "a", encoding="utf-8") as f:
            f.write(self.txt)
        self.txt = ""    # reset to nullstring, since we appended


if __name__ == "__main__":
    test = RestartTest()
    test.run()